//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Descriptor/VectorProperty.cpp
//! @brief     Implements class VectorProperty
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Descriptor/VectorProperty.h"
#include "GUI/Support/XML/Serialize.h"
#include "GUI/Support/XML/UtilXML.h"
#include <QUuid>

using std::variant;

void VectorProperty::init(const QString& label, const QString& tooltip,
                          const variant<QString, Unit>& unit, const QString& persistentTag)
{
    m_persistentTag = persistentTag;
    m_uid = QUuid::createUuid().toString();

    m_descriptor.init(label, tooltip, &m_value, unit);
    m_descriptor.uid = [this] { return m_uid; };
}


void Serialize::rwProperty(Streamer& s, VectorProperty& d)
{
    if (QXmlStreamWriter* w = s.xmlWriter()) {
        w->writeStartElement(d.persistentTag());
        GUI::Session::XML::writeAttribute(w, XML::Tags::Value, d.get());
        GUI::Session::XML::writeUid(w, d.uid());
        w->writeEndElement();
    } else if (QXmlStreamReader* r = s.xmlReader()) {
        r->readNextStartElement();
        s.assertCurrentTag(d.persistentTag());
        R3 vec;
        QString uid;
        GUI::Session::XML::readAttribute(r, XML::Tags::Value, &vec);
        GUI::Session::XML::readAttribute(r, XML::Tags::Id, &uid);
        d.set(vec);
        d.setUid(uid);
        s.gotoEndElementOfTag(d.persistentTag());
    } else
        ASSERT(0);
}
