//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/InstrumentCollection.h
//! @brief     Defines class InstrumentCollection
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DEVICE_INSTRUMENTCOLLECTION_H
#define BORNAGAIN_GUI_MODEL_DEVICE_INSTRUMENTCOLLECTION_H

#include <QStringList>
#include <QVector>
#include <functional>
#include <memory>

class InstrumentItem;
class Instrument2DItem;
class Streamer;

class InstrumentCollection {
public:
    ~InstrumentCollection();

    template <typename T>
    T* addInstrument()
    {
        auto* t = new T();
        m_instruments << t;
        return t;
    }

    void removeInstrument(InstrumentItem* instrument);
    void clear();

    void serialize(Streamer& s);

    void emplace_back(InstrumentItem* item) { m_instruments.push_back(item); } // TODO Qt6 emplace

    //! Inserts a deep copy (also of any non xml data in a pointwise axis)
    //! The id will not be copied, but a new unique one will be created
    //! Returns the newly created instrument.
    InstrumentItem* insertCopy(const InstrumentItem& source);

    QVector<InstrumentItem*> collectedItems() const;
    QVector<Instrument2DItem*> instrument2DItems() const;
    QVector<InstrumentItem*>
    instrumentItems(const std::function<bool(const InstrumentItem*)>& accept) const;

    InstrumentItem* findInstrumentById(const QString& instrumentId) const;
    bool instrumentExists(const QString& instrumentId) const;

    QString suggestInstrumentName(const QString& baseName) const;
    QStringList instrumentNames() const;

private:
    QVector<InstrumentItem*> m_instruments;
};

#endif // BORNAGAIN_GUI_MODEL_DEVICE_INSTRUMENTCOLLECTION_H
