//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/ResolutionFunctionItems.h
//! @brief     Defines family of ResolutionFunctionItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_DEVICE_RESOLUTIONFUNCTIONITEMS_H
#define BORNAGAIN_GUI_MODEL_DEVICE_RESOLUTIONFUNCTIONITEMS_H

#include "GUI/Model/Descriptor/DoubleProperty.h"
#include <memory>
#include <variant>

class IResolutionFunction2D;
class Streamer;

class ResolutionFunctionItem {
public:
    virtual ~ResolutionFunctionItem() = default;

    virtual std::unique_ptr<IResolutionFunction2D>
    createResolutionFunction(double scale = 1.0) const = 0;

    //! Set the unit of the distributed value
    virtual void setUnit(const std::variant<QString, Unit>&) {}
    virtual void serialize(Streamer&){};
};

class ResolutionFunctionNoneItem : public ResolutionFunctionItem {
public:
    std::unique_ptr<IResolutionFunction2D>
    createResolutionFunction(double scale = 1.0) const override;
};

class ResolutionFunction2DGaussianItem : public ResolutionFunctionItem {
public:
    ResolutionFunction2DGaussianItem();
    std::unique_ptr<IResolutionFunction2D>
    createResolutionFunction(double scale = 1.0) const override;

    void setUnit(const std::variant<QString, Unit>& unit) override;
    void serialize(Streamer& s) override;

    DOUBLE_PROPERTY(sigmaX, SigmaX);
    DOUBLE_PROPERTY(sigmaY, SigmaY);
};

#endif // BORNAGAIN_GUI_MODEL_DEVICE_RESOLUTIONFUNCTIONITEMS_H
