//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Job/FitParameterLinkItem.h
//! @brief     Defines class FitParameterLinkItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_JOB_FITPARAMETERLINKITEM_H
#define BORNAGAIN_GUI_MODEL_JOB_FITPARAMETERLINKITEM_H

#include <QObject>

class QXmlStreamReader;
class QXmlStreamWriter;

//! Holds link value, accessible from FitparameterModel.
class LinkItem : public QObject {
public:
    explicit LinkItem(QObject* parent = nullptr);

    QString link() const;
    void setLink(QString link);

    void writeTo(QXmlStreamWriter* writer) const;
    void readFrom(QXmlStreamReader* reader);

private:
    QString m_link;
};


//! Holds a link to ParameterItem in tuning tree.
class FitParameterLinkItem : public QObject {
public:
    explicit FitParameterLinkItem(QObject* parent = nullptr);

    QString link() const;
    void setLink(const QString& link);

    QString title() const;
    void setTitle(const QString& title);
    LinkItem* linkItem() const;

    void writeTo(QXmlStreamWriter* writer) const;
    void readFrom(QXmlStreamReader* reader);

private:
    QString m_title;
    LinkItem* m_linkItem;
};

#endif // BORNAGAIN_GUI_MODEL_JOB_FITPARAMETERLINKITEM_H
