//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/LayerItem.h
//! @brief     Defines class LayerItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_LAYERITEM_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_LAYERITEM_H

#include "GUI/Model/Descriptor/DoubleProperty.h"
#include "GUI/Model/Descriptor/SelectionDescriptor.h"
#include "GUI/Model/Descriptor/UIntProperty.h"
#include "GUI/Model/Sample/Item3D.h"
#include "GUI/Model/Sample/ItemWithMaterial.h"
#include "GUI/Model/Sample/LayerRoughnessItems.h"
#include <QColor>
#include <QUuid>
#include <QVector>
#include <memory>
#include <variant>

class LayerBasicRoughnessItem;
class ParticleLayoutItem;
class ItemWithParticles;
class QXmlStreamReader;
class QXmlStreamWriter;
class Streamer;

class LayerItem : public virtual ItemWithMaterial, public virtual Item3D {
public:
    explicit LayerItem(const MaterialItems* materials);

    void setLayerName(const QString& name);
    QString layerName() const;

    QVector<ItemWithMaterial*> itemsWithMaterial();
    QVector<ItemWithParticles*> itemsWithParticles() const;

    DoubleDescriptor thickness() const;

    SelectionDescriptor<LayerBasicRoughnessItem*> roughness();
    void setBasicRoughness();
    void clearRoughness();

    UIntDescriptor numSlices() const;

    QVector<ParticleLayoutItem*> layouts() const;
    ParticleLayoutItem* addLayout();
    void removeLayout(ParticleLayoutItem* layout);

    QColor color() const;
    void setColor(const QColor& color);

    void setIsTopLayer(bool b);
    bool isTopLayer() const;

    void setIsBottomLayer(bool b);
    bool isBottomLayer() const;

    void serialize(Streamer& s);

private:
    QString m_name;
    UIntProperty m_numSlices;
    DoubleProperty m_thickness;
    QColor m_color;
    bool m_isTopLayer = false;
    bool m_isBottomLayer = false;
    QVector<ParticleLayoutItem*> m_layouts;
    std::unique_ptr<LayerBasicRoughnessItem> m_topRoughness;
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_LAYERITEM_H
